@@ -40,26 +40,27 @@ static unsigned short* ticonv_nonusb_to_utf16(const unsigned long * charset, con
 {
 	const unsigned char *p = (const unsigned char *)ti;
 	unsigned short *q = utf16;
-	unsigned long c;
 
-	if (p != NULL && q != NULL)
+	if (ti == NULL || utf16 == NULL)
 	{
-		while (*p)
-		{
-			c=charset[*(p++)];
-			if (c<0x10000)
-			{
-				*(q++)=(unsigned short)c;
-			}
-			else
-			{
-				*(q++)=(unsigned short)(c>>16);
-				*(q++)=(unsigned short)(c&0xffff);
-			}
-		}
-		*q=0;
+		return NULL;
 	}
 
+	while (*p)
+	{
+		unsigned long c = charset[*(p++)];
+		if (c < 0x10000)
+		{
+			*(q++) = (unsigned short)c;
+		}
+		else
+		{
+			*(q++) = (unsigned short)(c >> 16);
+			*(q++) = (unsigned short)(c & 0xffff);
+		}
+	}
+	*q = 0;
+
 	return utf16;
 }
 
@@ -67,51 +68,134 @@ static unsigned short * ticonv_usb_to_utf16(const char *ti, unsigned short *utf1
 {
 	unsigned short *tmp;
 
+	if (ti == NULL || utf16 == NULL)
+	{
+		return NULL;
+	}
+
 	tmp = ticonv_utf8_to_utf16(ti);
+	if (tmp == NULL)
+	{
+		return NULL;
+	}
+
 	if (utf16 != NULL)
 	{
 		memcpy(utf16, tmp, 2 * ticonv_utf16_strlen(tmp));
 	}
-	g_free(tmp);
+	ticonv_utf16_free(tmp);
 
 	return utf16;
 }
 
+static char * ticonv_utf16_to_nonusb(const unsigned long * charset, const unsigned short *utf16, char *ti)
+{
+	const unsigned short *p = utf16;
+	unsigned char *q = (unsigned char *)ti;
+
+	if (utf16 == NULL || ti == NULL)
+	{
+		return NULL;
+	}
+
+	while (*p)
+	{
+		unsigned long c = *(p++);
+
+		if ((c & 0xfc00) == 0xd800)
+		{
+			c = (c << 16) | *(p++);
+		}
+
+		if (c < 256 && charset[c] == c)
+		{
+			unsigned int i;
+
+			if (c == 'x' && *p == 0x0305) // x followed by combining overline.
+			{
+				for (i = 0; i < 256; i++) {
+					if (charset[i] == 0x00780305) {
+						c = i; // xbar has that character code.
+						break;
+					}
+				}
+				p++;
+			}
+			else if (c == 'y' && *p == 0x0305) // y followed by combining overline.
+			{
+				for (i = 0; i < 256; i++) {
+					if (charset[i] == 0x00790305) {
+						c = i; // ybar has that character code.
+						break;
+					}
+				}
+				p++;
+			}
+
+			*(q++) = c;
+		}
+		else
+		{
+			unsigned int i;
+			*q = '?';
+			for (i = 0; i < 256; i++) {
+				if (charset[i] == c) {
+					*q = i;
+					break;
+				}
+			}
+			q++;
+		}
+	}
+	*q = 0;
+
+	return ti;
+}
+
 static char* ticonv_utf16_to_usb(const unsigned short *utf16, char *ti)
 {
 	char *tmp;
 
+	if (utf16 == NULL || ti == NULL)
+	{
+		return NULL;
+	}
+
 	tmp = ticonv_utf16_to_utf8(utf16);
+	if (tmp == NULL)
+	{
+		return NULL;
+	}
+
 	if (ti != NULL)
 	{
 		strcpy(ti, tmp);
 	}
-	g_free(tmp);
+	ticonv_utf8_free(tmp);
 
 	return ti;
 }
 
-
 ///////////// TI89,92,92+,V200,Titanium /////////////
 
 TIEXPORT4 const unsigned long TICALL ti9x_charset[256] =
 {
 // control chars
- 0,
- 1,
- 2,
- 3,
- 4,
- 5,
- 6,
- 7,
- 8,
- 9,
- 10,
+ 0, // 0x2592 is prettier
+ 1, // 0x2401 is prettier
+ 2, // 0x2402 is prettier
+ 3, // 0x2403 is prettier
+ 4, // 0x2404 is prettier
+ 5, // 0x2405 is prettier
+ 6, // 0x2406 is prettier
+ 7, // 0x2407, 0xd83ddd14 are prettier
+ 8, // 0x232b is prettier
+ 9, // 0x21e5 is prettier
+ 10, // 0x21b4 is prettier
  0x2934,
- 12,
- 13,
- 0x2693, // "locked" symbol (doesn't exist in Unicode) <-> anchor
+ 12, // 0x219f (upwards version of 0x21a1 form feed) is prettier
+ 13, // 0x21b5 is prettier
+ 0x2693, // For now, using anchor symbol U+2693 to represent locked, but should switch to 0xd83ddd12 (U+1F512) when Unicode 6.0+ is more widely implemented.
  0x2713,
  0x25fe,
  0x25c2,
@@ -130,119 +214,17 @@ TIEXPORT4 const unsigned long TICALL ti9x_charset[256] =
  0x2282,
  0x2208,
 // ASCII
- 32,
- 33,
- 34,
- 35,
- 36,
- 37,
- 38,
- 39,
- 40,
- 41,
- 42,
- 43,
- 44,
- 45,
- 46,
- 47,
- 48,
- 49,
- 50,
- 51,
- 52,
- 53,
- 54,
- 55,
- 56,
- 57,
- 58,
- 59,
- 60,
- 61,
- 62,
- 63,
- 64,
- 65,
- 66,
- 67,
- 68,
- 69,
- 70,
- 71,
- 72,
- 73,
- 74,
- 75,
- 76,
- 77,
- 78,
- 79,
- 80,
- 81,
- 82,
- 83,
- 84,
- 85,
- 86,
- 87,
- 88,
- 89,
- 90,
- 91,
- 92,
- 93,
- 94,
- 95,
- 96,
- 97,
- 98,
- 99,
- 100,
- 101,
- 102,
- 103,
- 104,
- 105,
- 106,
- 107,
- 108,
- 109,
- 110,
- 111,
- 112,
- 113,
- 114,
- 115,
- 116,
- 117,
- 118,
- 119,
- 120,
- 121,
- 122,
- 123,
- 124,
- 125,
- 126,
- 0x25c6,
+ 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
+ 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
+
+ 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
+ 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95,
+
+ 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
+ 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 0x25c6,
+
 // Greek letters
- 0x3b1,
- 0x3b2,
- 0x393,
- 0x3b3,
- 0x394,
- 0x3b4,
- 0x3b5,
- 0x3b6,
- 0x3b8,
- 0x3bb,
- 0x3be,
- 0x3a0,
- 0x3c0,
- 0x3c1,
- 0x3a3,
- 0x3c3,
+ 0x3b1, 0x3b2, 0x393, 0x3b3, 0x394, 0x3b4, 0x3b5, 0x3b6, 0x3b8, 0x3bb, 0x3be, 0x3a0, 0x3c0, 0x3c1, 0x3a3, 0x3c3,
  0x3c4,
  0x3c6,
  0x3c8,
@@ -253,7 +235,7 @@ TIEXPORT4 const unsigned long TICALL ti9x_charset[256] =
  0x212f, //e
  0xd875dc8a, //i (non-BMP character)
  0x2b3, //r
- 0x22ba, //T
+ 0x22ba, //T - questionable.
  0x00780305, //x bar (requires composition)
  0x00790305, //y bar (requires composition)
  0x2264,
@@ -281,7 +263,7 @@ TIEXPORT4 const unsigned long TICALL ti9x_charset[256] =
  177,
  178,
  179,
- 180, //^-1, but there is no such character in Unicode
+ 180, //^-1, but there is no such character in Unicode 0xB9 is superscript 1
  181,
  182,
  183,
@@ -293,310 +275,14 @@ TIEXPORT4 const unsigned long TICALL ti9x_charset[256] =
  0x222b,
  0x221e,
  191,
- 192,
- 193,
- 194,
- 195,
- 196,
- 197,
- 198,
- 199,
- 200,
- 201,
- 202,
- 203,
- 204,
- 205,
- 206,
- 207,
- 208,
- 209,
- 210,
- 211,
- 212,
- 213,
- 214,
- 215,
- 216,
- 217,
- 218,
- 219,
- 220,
- 221,
- 222,
- 223,
- 224,
- 225,
- 226,
- 227,
- 228,
- 229,
- 230,
- 231,
- 232,
- 233,
- 234,
- 235,
- 236,
- 237,
- 238,
- 239,
- 240,
- 241,
- 242,
- 243,
- 244,
- 245,
- 246,
- 247,
- 248,
- 249,
- 250,
- 251,
- 252,
- 253,
- 254,
- 255 
+
+ 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207,
+ 208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223,
+
+ 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239,
+ 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255
 };
 
-/**
- * ticonv_utf16_to_ti9x:
- * @utf16: null terminated string (input)
- * @ti: null terminated string (output)
- *
- * UTF-16 to TI89,92,92+,V200,Titanium charset conversion.
- *
- * Return value: returns the destination pointer or NULL if error.
- **/
-TIEXPORT4 char* TICALL ticonv_utf16_to_ti9x(const unsigned short *utf16, char *ti)
-{
-	const unsigned short *p = utf16;
-	unsigned char *q = (unsigned char *)ti;
-
-	if (utf16 == NULL || ti == NULL)
-	{
-		// Something's wrong with the arguments !
-		return NULL;
-	}
-
-	while (*p) {
-		if (   *p<=10
-		    || *p==12 || *p==13
-		    || (*p>=32 && *p<=119)
-		    || (*p>=122 && *p<=126)
-		    || (*p>=161 && *p<=167)
-		    || (*p>=169 && *p<=172)
-		    || (*p>=174 && *p<=183)
-		    || (*p>=185 && *p<=187)
-		    || (*p>=191 && *p<=255)
-		   ) {
-			*(q++)=(unsigned char)*(p++);
-		} else {
-			switch (*(p++)) {
-				case 0x2934:
-					*(q++)=11;
-					break;
-				case 0x2693:
-					*(q++)=14;
-					break;
-				case 0x2713:
-					*(q++)=15;
-					break;
-				case 0x25fe:
-					*(q++)=16;
-					break;
-				case 0x25c2:
-					*(q++)=17;
-					break;
-				case 0x25b8:
-					*(q++)=18;
-					break;
-				case 0x25b4:
-					*(q++)=19;
-					break;
-				case 0x25be:
-					*(q++)=20;
-					break;
-				case 0x2190:
-					*(q++)=21;
-					break;
-				case 0x2192:
-					*(q++)=22;
-					break;
-				case 0x2191:
-					*(q++)=23;
-					break;
-				case 0x2193:
-					*(q++)=24;
-					break;
-				case 0x25c0:
-					*(q++)=25;
-					break;
-				case 0x25b6:
-					*(q++)=26;
-					break;
-				case 0x2b06:
-					*(q++)=27;
-					break;
-				case 0x222a:
-					*(q++)=28;
-					break;
-				case 0x2229:
-					*(q++)=29;
-					break;
-				case 0x2282:
-					*(q++)=30;
-					break;
-				case 0x2208:
-					*(q++)=31;
-					break;
-				case 0x25c6:
-					*(q++)=127;
-					break;
-				case 0x3b1:
-					*(q++)=128;
-					break;
-				case 0x3b2:
-					*(q++)=129;
-					break;
-				case 0x393:
-					*(q++)=130;
-					break;
-				case 0x3b3:
-					*(q++)=131;
-					break;
-				case 0x394:
-					*(q++)=132;
-					break;
-				case 0x3b4:
-					*(q++)=133;
-					break;
-				case 0x3b5:
-					*(q++)=134;
-					break;
-				case 0x3b6:
-					*(q++)=135;
-					break;
-				case 0x3b8:
-					*(q++)=136;
-					break;
-				case 0x3bb:
-					*(q++)=137;
-					break;
-				case 0x3be:
-					*(q++)=138;
-					break;
-				case 0x3a0:
-					*(q++)=139;
-					break;
-				case 0x3c0:
-					*(q++)=140;
-					break;
-				case 0x3c1:
-					*(q++)=141;
-					break;
-				case 0x3a3:
-					*(q++)=142;
-					break;
-				case 0x3c3:
-					*(q++)=143;
-					break;
-				case 0x3c4:
-					*(q++)=144;
-					break;
-				case 0x3c6:
-					*(q++)=145;
-					break;
-				case 0x3c8:
-					*(q++)=146;
-					break;
-				case 0x3a9:
-					*(q++)=147;
-					break;
-				case 0x3c9:
-					*(q++)=148;
-					break;
-				case 0x212f:
-					*(q++)=150;
-					break;
-				case 0x2b3:
-					*(q++)=152;
-					break;
-				case 0x22ba:
-					*(q++)=153;
-					break;
-				case 0x2264:
-					*(q++)=156;
-					break;
-				case 0x2260:
-					*(q++)=157;
-					break;
-				case 0x2265:
-					*(q++)=158;
-					break;
-				case 0x2220:
-					*(q++)=159;
-					break;
-				case 0x2026:
-					*(q++)=160;
-					break;
-				case 0x221a:
-					*(q++)=168;
-					break;
-				case 0x2212:
-					*(q++)=173;
-					break;
-				case 0x207a:
-					*(q++)=184;
-					break;
-				case 0x2202:
-					*(q++)=188;
-					break;
-				case 0x222b:
-					*(q++)=189;
-					break;
-				case 0x221e:
-					*(q++)=190;
-					break;
-				case 0x78:
-					if (*p==0x0305) {
-						*(q++)=154; // xbar
-						p++;
-					} else {
-						*(q++)='x';
-					}
-					break;
-				case 0x79:
-					if (*p==0x0305) {
-						*(q++)=155; // ybar
-						p++;
-					} else {
-						*(q++)='y';
-					}
-					break;
-				case 0xd875:
-					if (*p==0xdda4) {
-						*(q++)=149;
-						p++;
-						break;
-					} else if (*p==0xdc8a) {
-						*(q++)=151;
-						p++;
-						break;
-					}
-				default:
-					if (p[-1] >= 0xd800 && p[-1] <= 0xdbff)
-						p++;
-					*(q++)='?';
-					break;
-			}
-		}
-	}
-	*q=0;
-
-	return ti;
-}
-
 /**
  * ticonv_ti9x_to_utf16:
  * @ti: null terminated string (input)
@@ -611,6 +297,20 @@ TIEXPORT4 unsigned short* TICALL ticonv_ti9x_to_utf16(const char *ti, unsigned s
 	return ticonv_nonusb_to_utf16(ti9x_charset, ti, utf16);
 }
 
+/**
+ * ticonv_utf16_to_ti9x:
+ * @utf16: null terminated string (input)
+ * @ti: null terminated string (output)
+ *
+ * UTF-16 to TI89,92,92+,V200,Titanium charset conversion.
+ *
+ * Return value: returns the destination pointer or NULL if error.
+ **/
+TIEXPORT4 char* TICALL ticonv_utf16_to_ti9x(const unsigned short *utf16, char *ti)
+{
+	return ticonv_utf16_to_nonusb(ti9x_charset, utf16, ti);
+}
+
 ///////////// TI73 /////////////
 
 TIEXPORT4 const unsigned long TICALL ti73_charset[256] = {
@@ -638,8 +338,8 @@ TIEXPORT4 const unsigned long TICALL ti73_charset[256] = {
 	'p',    'q',    'r',    's',    't',    'u',    'v',    'w',
 	'x',    'y',    'z',    '{',    '|',    '}',    '~',    '=',
 
-	0x2080,	0x2081, 0x2082, 0x2083, 0x2084,	0x2085,	0x2086,	0x2087,
-	0x2088,	0x2089,	192+1,  192+0,  192+2,  192+4,  224+1,  224+0,
+	0x2080, 0x2081, 0x2082, 0x2083, 0x2084, 0x2085, 0x2086, 0x2087,
+	0x2088, 0x2089, 192+1,  192+0,  192+2,  192+4,  224+1,  224+0,
 
 	224+2,  224+4,  200+1,  200+0,  200+2,  200+4,  231+1,  231+0,
 	231+2,  231+4,  204+1,  204+0,  204+2,  204+3,  236+1,  236+0,
@@ -684,17 +384,17 @@ TIEXPORT4 unsigned short* TICALL ticonv_ti73_to_utf16(const char *ti, unsigned s
  *
  * UTF-16 to TI73 charset conversion.
  *
- * Return value: empty string (not implemented)
+ * Return value: NULL (not implemented)
  **/
 TIEXPORT4 char* TICALL ticonv_utf16_to_ti73(const unsigned short *utf16, char *ti)
 {
-	return (char *)"";
+	return ticonv_utf16_to_nonusb(ti73_charset, utf16, ti);
 }
 
 ///////////// TI80 /////////////
 
-TIEXPORT4 const unsigned long TICALL ti80_charset[256] = { 
-	' ',    0x2588, '_',    0x2191, 'A',    0x25b6, '%',    '(',
+TIEXPORT4 const unsigned long TICALL ti80_charset[256] = {
+	' ',    0x2588, '_',    0x2191, 'A',    0x25b6, '%',    '(', // 0x00-0x7F
 	')',    '\"',   ',',    '!',    176,    '\'',   0x2b3,  180,
 
 	178,    'a',    'b',    'c',    'd',    'e',    'n',    'r', 
@@ -718,7 +418,7 @@ TIEXPORT4 const unsigned long TICALL ti80_charset[256] = {
 	'_',    '_',    '_',    '_',    '_',    0x2592, 0x2af0, 179,
 	'_',    '_',    '_',    '_',    '_',    '_',    '_',    '_',
 
-	' ',    0x2588, '_',    0x2191, 'A',    0x25b6, '%',    '(',
+	' ',    0x2588, '_',    0x2191, 'A',    0x25b6, '%',    '(', // 0x80-0xFF onwards: same as 0x00-0x7F
 	')',    '\"',   ',',    '!',    176,    '\'',   0x2b3,  180,
 
 	178,    'a',    'b',    'c',    'd',    'e',    'n',    'r', 
@@ -768,12 +468,12 @@ TIEXPORT4 unsigned short* TICALL ticonv_ti80_to_utf16(const char *ti, unsigned s
  **/
 TIEXPORT4 char* TICALL ticonv_utf16_to_ti80(const unsigned short *utf16, char *ti)
 {
+// TODO investigate reimplementation based on ticonv_utf16_to_nonusb().
 	const unsigned short *p = utf16;
 	unsigned char *q = (unsigned char *)ti;
 
 	if (utf16 == NULL || ti == NULL)
 	{
-		// Something's wrong with the arguments !
 		return NULL;
 	}
 
@@ -886,11 +586,11 @@ TIEXPORT4 unsigned short* TICALL ticonv_ti82_to_utf16(const char *ti, unsigned s
  *
  * UTF-16 to TI82 charset conversion.
  *
- * Return value: empty string (not implemented)
+ * Return value: NULL (not implemented)
  **/
 TIEXPORT4 char* TICALL ticonv_utf16_to_ti82(const unsigned short *utf16, char *ti)
 {
-	return (char *)"";
+	return ticonv_utf16_to_nonusb(ti82_charset, utf16, ti);
 }
 
 ///////////// TI83 /////////////
@@ -966,11 +666,11 @@ TIEXPORT4 unsigned short* TICALL ticonv_ti83_to_utf16(const char *ti, unsigned s
  *
  * UTF-16 to TI83 charset conversion.
  *
- * Return value: empty string (not implemented)
+ * Return value: NULL (not implemented)
  **/
 TIEXPORT4 char* TICALL ticonv_utf16_to_ti83(const unsigned short *utf16, char *ti)
 {
-	return (char *)"";
+	return ticonv_utf16_to_nonusb(ti83_charset, utf16, ti);
 }
 
 ///////////// TI83+/84+ /////////////
@@ -1013,10 +713,10 @@ TIEXPORT4 const unsigned long TICALL ti83p_charset[256] = {
 	0x0a8,  0x0bf,  0x0a1,  0x3b1,  0x3b2,  0x3b3,  0x394,  0x3b4,
 
 	0x3b5,  '[',    0x3bb,  0x3bc,  0x3c0,  0x3c1,  0x3a3,  0x3c3,
-	0x3c4,  0x3d5,  0x3a9,  'x',    'y',    0xa4,   0x2026, 0x25c0,
+	0x3c4,  0x3d5,  0x3a9,  0x00780305, 0x00790305,    0xa4,   0x2026, 0x25c0,
 
 	0x25fe, '?',    0x2212, 178,    176,    179,    '\n',   0xd875dc8a,
-	'?',    0x3c7,  'F',    0x212f, 'L',   'N',     '_',	0x2192,
+	'?',    0x3c7,  'F',    0x212f, 'L',   'N',     '_',    0x2192,
 
 	'_',    '_',    '_',    '_',    '_',    '_',    '_',    '_',
 	'_',    '_',    '_',    '_',    '_',    '_',    '_',    '_',
@@ -1050,12 +750,12 @@ TIEXPORT4 unsigned short* TICALL ticonv_ti83p_to_utf16(const char *ti, unsigned
  **/
 TIEXPORT4 char* TICALL ticonv_utf16_to_ti83p(const unsigned short *utf16, char *ti)
 {
+// TODO investigate reimplementation based on ticonv_utf16_to_nonusb().
 	const unsigned short *p = utf16;
 	unsigned char *q = (unsigned char *)ti;
 
 	if (utf16 == NULL || ti == NULL)
 	{
-		// Something's wrong with the arguments !
 		return NULL;
 	}
 
@@ -1236,11 +936,11 @@ TIEXPORT4 unsigned short* TICALL ticonv_ti85_to_utf16(const char *ti, unsigned s
  *
  * UTF-16 to TI85 charset conversion.
  *
- * Return value: empty string (not implemented)
+ * Return value: NULL (not implemented)
  **/
 TIEXPORT4 char* TICALL ticonv_utf16_to_ti85(const unsigned short *utf16, char *ti)
 {
-	return (char *)"";
+	return ticonv_utf16_to_nonusb(ti85_charset, utf16, ti);
 }
 
 ///////////// TI86 /////////////
@@ -1262,7 +962,7 @@ TIEXPORT4 const unsigned long TICALL ti86_charset[256] = {
 	'H',    'I',    'J',    'K',    'L',    'M',    'N',    'O',
 
 	'P',    'Q',    'R',    'S',    'T',    'U',    'V',    'W',
-	'X',    'Y',    'Z',    '[',    '\\',   ']',    '^',    '_',	// TI82 != TI85: theta <-> [
+	'X',    'Y',    'Z',    '[',    '\\',   ']',    '^',    '_', // TI82 != TI85: theta <-> [
 
 	'`',    'a',    'b',    'c',    'd',    'e',    'f',    'g',
 	'h',    'i',    'j',    'k',    'l',    'm',    'n',    'o',
@@ -1283,7 +983,7 @@ TIEXPORT4 const unsigned long TICALL ti86_charset[256] = {
 	0x0a8,  0x0bf,  0x0a1,  0x3b1,  0x3b2,  0x3b3,  0x394,  0x3b4,
 
 	0x3b5,  0x3b8,  0x3bb,  0x3bc,  0x3c0,  0x3c1,  0x3a3,  0x3c3,
-	0x3c4,  0x3d5,  0x3a9,  'x',    'y',    '?',    0x2026, 0x25c0,
+	0x3c4,  0x3d5,  0x3a9,  0x00780305, 0x00790305,   '?',    0x2026, 0x25c0,
 
 	0x25fe, '?',    0x2212, 178,    176,    179,    '\n',   0x26b6,
 	'_',	'_',    '_',    '_',    '_',    '_',    '_',    '_',
@@ -1316,11 +1016,11 @@ TIEXPORT4 unsigned short* TICALL ticonv_ti86_to_utf16(const char *ti, unsigned s
  *
  * UTF-16 to TI86 charset conversion.
  *
- * Return value: empty string (not implemented)
+ * Return value: NULL (not implemented)
  **/
 TIEXPORT4 char* TICALL ticonv_utf16_to_ti86(const unsigned short *utf16, char *ti)
 {
-	return (char *)"";
+	return ticonv_utf16_to_nonusb(ti86_charset, utf16, ti);
 }
 
 ///////////// TI84+ USB & Titanium USB /////////////
