@@ -37,65 +37,14 @@
 #include "ticonv.h"
 #include "charset.h"
 
-#define TIE		TIEXPORT4
-#define TIC		TICALL
-
-
-/* This is a version of tifiles_string_to_model without the "USB" variants
-   which don't make much sense for charset use. And we can't use tifiles
-   functions here anyway (otherwise we'd have a circular dependency). */
-static CalcModel ticonv_string_to_model(const char *str)
-{
-	if(!g_ascii_strcasecmp(str, "TI73") || !g_ascii_strcasecmp(str, "73"))
-		return CALC_TI73;
-	else if(!g_ascii_strcasecmp(str, "TI80") || !g_ascii_strcasecmp(str, "80"))
-		return CALC_TI80;
-	else if(!g_ascii_strcasecmp(str, "TI82") || !g_ascii_strcasecmp(str, "82"))
-		return CALC_TI82;
-	else if(!g_ascii_strcasecmp(str, "TI83") || !g_ascii_strcasecmp(str, "83"))
-		return CALC_TI83;
-	else if(   !g_ascii_strncasecmp(str, "TI83+", 5)
-		|| !g_ascii_strncasecmp(str, "TI83p", 5)
-		|| !g_ascii_strncasecmp(str, "83+", 3)
-		|| !g_ascii_strncasecmp(str, "83p", 3)
-	       )
-		return CALC_TI83P;
-	else if(   !g_ascii_strncasecmp(str, "TI84+", 5)
-		|| !g_ascii_strncasecmp(str, "TI84p", 5)
-		|| !g_ascii_strncasecmp(str, "84+", 3)
-		|| !g_ascii_strncasecmp(str, "84p", 3)
-	       )
-		return CALC_TI84P;
-	else if(!g_ascii_strcasecmp(str, "TI85") || !g_ascii_strcasecmp(str, "85"))
-		return CALC_TI85;
-	else if(!g_ascii_strcasecmp(str, "TI86") || !g_ascii_strcasecmp(str, "86"))
-		return CALC_TI86;
-	else if(!g_ascii_strcasecmp(str, "TI89") || !g_ascii_strcasecmp(str, "89"))
-		return CALC_TI89;
-	else if(!g_ascii_strcasecmp(str, "TI89t") || !g_ascii_strcasecmp(str, "89t"))
-		return CALC_TI89T;
-	else if(!g_ascii_strcasecmp(str, "TI92") || !g_ascii_strcasecmp(str, "92"))
-		return CALC_TI92;
-	else if(   !g_ascii_strncasecmp(str, "TI92+", 5)
-		|| !g_ascii_strncasecmp(str, "TI92p", 5)
-		|| !g_ascii_strncasecmp(str, "92+", 3)
-		|| !g_ascii_strncasecmp(str, "92p", 3)
-	       )
-		return CALC_TI92P;
-	else if(!g_ascii_strcasecmp(str, "V200") || !g_ascii_strcasecmp(str, "V200"))
-		return CALC_V200;
-		
-	return CALC_NONE;
-}
-
 /* Allocate descriptor for code conversion from codeset FROMCODE to
    codeset TOCODE.  */
-TIE ticonv_iconv_t TIC ticonv_iconv_open (const char *tocode, const char *fromcode)
+TIEXPORT4 ticonv_iconv_t TICALL ticonv_iconv_open (const char *tocode, const char *fromcode)
 {
   ticonv_iconv_t cd;
   cd.src_calc=ticonv_string_to_model(fromcode);
   cd.dest_calc=ticonv_string_to_model(tocode);
-  cd.iconv_desc=iconv_open(cd.src_calc?"UTF-16":fromcode,cd.dest_calc?"UTF-16":tocode);
+  cd.iconv_desc=iconv_open(cd.src_calc || fromcode == NULL ? "UTF-16" : fromcode, cd.dest_calc || tocode == NULL ? "UTF-16" : tocode);
   cd.lookahead=0;
   cd.lossy_count=0;
   return cd;
@@ -104,20 +53,23 @@ TIE ticonv_iconv_t TIC ticonv_iconv_open (const char *tocode, const char *fromco
 /* Convert at most *INBYTESLEFT bytes from *INBUF according to the
    code conversion algorithm specified by CD and place up to
    *OUTBYTESLEFT bytes in buffer at *OUTBUF.  */
-TIE size_t TIC ticonv_iconv (ticonv_iconv_t cd, char **__restrict inbuf,
+TIEXPORT4 size_t TICALL ticonv_iconv (ticonv_iconv_t cd, char **__restrict inbuf,
                              size_t *__restrict inbytesleft,
                              char **__restrict outbuf,
                              size_t *__restrict outbytesleft)
 {
   size_t result;
-  if (!inbuf || !inbuf) {
+  if (!inbytesleft || !outbytesleft) {
+    return 0;
+  }
+  if (!inbuf || !*inbuf) {
     if (!outbuf || !*outbuf || cd.dest_calc) {
       result=iconv(cd.iconv_desc,NULL,NULL,NULL,NULL)+cd.lossy_count;
       cd.lookahead=0;
       cd.lossy_count=0;
       return result;
     } else {
-      return iconv(cd.iconv_desc,NULL,NULL,outbuf,outbytesleft)+cd.lossy_count;
+      result=iconv(cd.iconv_desc,NULL,NULL,outbuf,outbytesleft)+cd.lossy_count;
       cd.lookahead=0;
       cd.lossy_count=0;
       return result;
@@ -143,7 +95,7 @@ TIE size_t TIC ticonv_iconv (ticonv_iconv_t cd, char **__restrict inbuf,
         temp=g_realloc(temp,(tempsize+temp2size)<<1);
         memcpy(temp+tempsize,temp2,temp2size);
         tempsize+=temp2size;
-        g_free(temp2);
+        ticonv_utf16_free(temp2);
       }
       g_free(input);
       iconv_src=temp;
@@ -268,10 +220,10 @@ TIE size_t TIC ticonv_iconv (ticonv_iconv_t cd, char **__restrict inbuf,
           if (!*p) *p='_';
         }
         l1=strlen(tmp1);
-        g_free(tmp1);
+        ticonv_ti_free(tmp1);
         tmp2=ticonv_charset_utf16_to_ti(cd.src_calc,temp);
         l2=strlen(tmp2);
-        g_free(tmp2);
+        ticonv_ti_free(tmp2);
         *inbuf+=l2-l1;
         *inbytesleft-=l2-l1;
       } else {
@@ -282,13 +234,13 @@ TIE size_t TIC ticonv_iconv (ticonv_iconv_t cd, char **__restrict inbuf,
       *inbuf=iconv_src;
       *inbytesleft=iconv_inbytes;
     }
-    g_free(temp);
+    ticonv_utf16_free(temp);
     return result;
   }
 }
 
 /* Free resources allocated for descriptor CD for code conversion.  */
-TIE int TIC ticonv_iconv_close (ticonv_iconv_t cd)
+TIEXPORT4 int TICALL ticonv_iconv_close (ticonv_iconv_t cd)
 {
   return iconv_close(cd.iconv_desc);
 }
